"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const child_process_1 = require("child_process");
const fs_1 = require("fs");
const fsPath = require("path");
const parseKeys_1 = require("./parseKeys");
const fuzzy_1 = require("./fuzzy");
class InputError extends Error {
    constructor(title, message) {
        super(message);
        this.title = title;
        this.message = message;
    }
}
function isSubCommand(args) {
    const isDashDashSubCommand = [
        '--list',
        '--version',
        '-l',
        '-v',
        '-h'
    ].includes(args[0]);
    const isMultiArgumentSubCommand = (args.length > 1
        && ['new', 'delete', 'rename'].includes(args[0]));
    return isDashDashSubCommand || isMultiArgumentSubCommand;
}
function executeSubCommand(name, args) {
    switch (name) {
        case '--list':
        case '-l': {
            listSubCommand();
            break;
        }
        case '--version':
        case '-v': {
            versionSubCommand();
            break;
        }
        case '-h': {
            helpSubCommand();
            break;
        }
        case 'new': {
            newSubCommand(args);
            break;
        }
        case 'rename': {
            renameSubCommand(args);
            break;
        }
        case 'delete': {
            deleteSubCommand(args);
            break;
        }
        default: {
            throw new InputError(`Unknown command ${bold(`git jump ${name}`)}`, `See ${bold('git jump --help')} for the list of supported commands.`);
        }
    }
}
function versionSubCommand() {
    process.stdout.write(`${readVersion()}\n`);
    process.exit(0);
}
function listSubCommand() {
    state.isInteractive = false;
    view(state);
    process.exit(0);
}
function newSubCommand(args) {
    const { status, message } = gitCommand('switch', ['-c', ...args]);
    state.scene = Scene.Message;
    state.message = message;
    if (status === 0) {
        updateBranchLastSwitch(args[0], Date.now(), state);
    }
    view(state);
    process.exit(status);
}
function helpSubCommand() {
    let help = fs_1.readFileSync(fsPath.join(__dirname, '../help.txt')).toString();
    help = help.replace(/\{bold\}(.+)\{\/bold\}/g, (substring, content) => bold(content));
    help = help.replace(/\{dim\}(.+)\{\/dim\}/g, (substring, content) => dim(content));
    help = help.replace(/\{wrap:(\d+)\}(.+)\{\/wrap\}/g, (substring, paddingSize, content) => {
        return multilineTextLayout(content.trim(), process.stdout.columns - parseInt(paddingSize)).map((line, index) => {
            return index === 0 ? line : ' '.repeat(paddingSize) + line;
        }).join('\n');
    });
    process.stdout.write(help);
    process.exit(0);
}
function renameSubCommand(args) {
    if (args.length < 2) {
        throw new InputError('Wrong Format.', `You should specify both current and new branch name, ${bold('git jump rename <old branch name> <new branch name>')}.`);
    }
    const { status, message } = gitCommand('branch', ['--move', args[0], args[1]]);
    state.scene = Scene.Message;
    state.message = message;
    if (status === 0) {
        renameJumpDataBranch(args[0], args[1], state);
        state.message.push('Renamed.');
    }
    view(state);
    process.exit(status);
}
function deleteSubCommand(args) {
    const { status, message } = gitCommand('branch', ['--delete', ...args]);
    state.scene = Scene.Message;
    state.message = message;
    if (status === 0) {
        deleteJumpDataBranch(args, state);
    }
    view(state);
    process.exit(status);
}
var ListLineType;
(function (ListLineType) {
    ListLineType[ListLineType["Head"] = 0] = "Head";
    ListLineType[ListLineType["Branch"] = 1] = "Branch";
})(ListLineType || (ListLineType = {}));
var Scene;
(function (Scene) {
    Scene[Scene["List"] = 0] = "List";
    Scene[Scene["Message"] = 1] = "Message";
})(Scene || (Scene = {}));
const state = {
    rows: process.stdout.rows,
    columns: process.stdout.columns,
    highlightedLineIndex: 0,
    maxRows: process.stdout.rows,
    branches: [],
    searchString: '',
    searchStringCursorPosition: 0,
    currentHEAD: {
        detached: false,
        sha: null,
        branchName: null
    },
    list: [],
    lineSelected: false,
    scene: Scene.List,
    message: [],
    gitRepoFolder: null,
    isInteractive: true,
    latestPackageVersion: null
};
function dim(s) {
    return `\x1b[2m${s}\x1b[22m`;
}
function bold(s) {
    return `\x1b[1m${s}\x1b[22m`;
}
function highlight(s) {
    return `\x1b[38;5;4m${s}\x1b[39m`;
}
function green(s) {
    return `\x1b[38;5;2m${s}\x1b[39m`;
}
function yellow(s) {
    return `\x1b[38;5;3m${s}\x1b[39m`;
}
function red(s) {
    return `\x1b[38;5;1m${s}\x1b[39m`;
}
function calculateLinesWindow(linesCount, highlightedLineIndex) {
    const windowSize = state.rows - 2;
    const windowHalf = Math.floor(windowSize / 2);
    const topIndex = Math.max(0, Math.min(linesCount - windowSize, state.highlightedLineIndex - windowHalf));
    const bottomIndex = topIndex + (windowSize - 1);
    return { topIndex, bottomIndex };
}
var LayoutColumnType;
(function (LayoutColumnType) {
    LayoutColumnType[LayoutColumnType["Index"] = 0] = "Index";
    LayoutColumnType[LayoutColumnType["BranchName"] = 1] = "BranchName";
    LayoutColumnType[LayoutColumnType["LastUsed"] = 2] = "LastUsed";
    LayoutColumnType[LayoutColumnType["MoreIndicator"] = 3] = "MoreIndicator";
})(LayoutColumnType || (LayoutColumnType = {}));
function calculateLayout(state) {
    const indexColumnWidth = 3;
    const moreIndicatorColumnWidth = 5;
    const branchNameColumnWidth = Math.min(state.columns - indexColumnWidth - moreIndicatorColumnWidth, Math.max.apply(null, state.branches.map((branch) => {
        return branch.name.length;
    })));
    const moreIndicatorSpacingWidth = state.columns - indexColumnWidth - branchNameColumnWidth - moreIndicatorColumnWidth;
    return [
        { type: LayoutColumnType.Index, width: indexColumnWidth },
        { type: LayoutColumnType.BranchName, width: branchNameColumnWidth },
        { type: LayoutColumnType.MoreIndicator, width: moreIndicatorSpacingWidth + moreIndicatorColumnWidth }
    ];
}
function highlightLine(line, lineIndex, highlightedLineIndex, selected = false) {
    if (lineIndex === highlightedLineIndex) {
        return selected ? green(line) : highlight(line);
    }
    return line;
}
function addScrollIndicator(line, lineIndex, listLength, listWindow, layout) {
    if (lineIndex === listWindow.bottomIndex && listWindow.bottomIndex < listLength - 1) {
        return line + dim('   ↓ '.padStart(layout[layout.length - 1].width, ' '));
    }
    return line;
}
function truncate(s, maxWidth) {
    let truncated = s.slice(0, maxWidth);
    if (truncated.length < s.length) {
        truncated = `${truncated.substring(0, truncated.length - 1)}…`;
    }
    return truncated;
}
function getQuickSelectLines(list) {
    return list.filter((line) => {
        return line.type !== ListLineType.Head;
    }).slice(0, 10);
}
const branchIndexPadding = '   ';
function viewCurrentHEAD(currentHEAD, layout) {
    return layout.reduce((line, column) => {
        if (column.type === LayoutColumnType.Index) {
            return line + branchIndexPadding;
        }
        if (column.type === LayoutColumnType.BranchName) {
            const branch = currentHEAD.detached
                ? `${bold(currentHEAD.sha)} ${dim('(detached)')}`
                : bold(currentHEAD.branchName);
            return line + branch;
        }
        return line;
    }, '');
}
function viewBranch(branch, index, layout) {
    return layout.reduce((line, column) => {
        if (column.type === LayoutColumnType.Index) {
            return line + (index < 10 ? ` ${dim(index.toString())} ` : branchIndexPadding);
        }
        if (column.type === LayoutColumnType.BranchName) {
            return line + truncate(branch.name, column.width).padEnd(column.width, ' ');
        }
        return line;
    }, '');
}
function viewListLines(state, layout) {
    let quickSelectIndex = -1;
    return state.list.map((line) => {
        switch (line.type) {
            case ListLineType.Head: {
                return viewCurrentHEAD(line.content, layout);
            }
            case ListLineType.Branch: {
                quickSelectIndex++;
                return viewBranch(line.content, quickSelectIndex, layout);
            }
        }
    });
}
function viewNonInteractiveList(state) {
    const layout = [
        { type: LayoutColumnType.BranchName, width: state.columns },
    ];
    return viewListLines(state, layout);
}
function viewList(state) {
    if (state.list.length === 0) {
        return [`${branchIndexPadding}${dim('No such branches')}`];
    }
    const layout = calculateLayout(state);
    const listWindow = calculateLinesWindow(state.list.length, state.highlightedLineIndex);
    return viewListLines(state, layout)
        .map((line, index) => {
        return addScrollIndicator(highlightLine(line, index, state.highlightedLineIndex), index, state.list.length, listWindow, layout);
    })
        .slice(listWindow.topIndex, listWindow.bottomIndex + 1);
}
function viewQuickSelectHint(maxIndex, columnWidth) {
    const trailingIndex = maxIndex > 0 ? `..${maxIndex}` : '';
    const modifierKey = os.type() === 'Darwin' ? '⌥' : 'Alt';
    return dim(`${modifierKey}+0${trailingIndex} quick select `.padStart(columnWidth, ' '));
}
function viewSearch(state, width) {
    const SEARCH_PLACEHOLDER = 'Search';
    return state.searchString === '' ? dim(SEARCH_PLACEHOLDER.padEnd(width, ' ')) : truncate(state.searchString, width).padEnd(width, ' ');
}
function viewSearchLine(state) {
    const searchPlaceholderWidth = 6;
    const searchWidth = Math.min(state.columns - branchIndexPadding.length, Math.max(state.searchString.length, searchPlaceholderWidth));
    const hintMinWidth = 25;
    let line = branchIndexPadding + viewSearch(state, searchWidth);
    const hintColumnWidth = state.columns - (branchIndexPadding.length + searchWidth);
    if (hintColumnWidth < hintMinWidth) {
        return line;
    }
    const quickSelectLines = getQuickSelectLines(state.list);
    if (quickSelectLines.length === 0) {
        return line;
    }
    line += viewQuickSelectHint(quickSelectLines.length - 1, hintColumnWidth);
    return line;
}
function view(state) {
    switch (state.scene) {
        case Scene.List: {
            if (!state.isInteractive) {
                render(viewNonInteractiveList(state).concat(['']));
                return;
            }
            let lines = [];
            lines.push(viewSearchLine(state));
            lines = lines.concat(viewList(state));
            clear();
            render(lines);
            cursorTo(branchIndexPadding.length + state.searchStringCursorPosition + 1, 1);
            break;
        }
        case Scene.Message: {
            clear();
            const lineSpacer = '  ';
            const lines = [
                '',
                ...state.message.reduce((lines, line) => {
                    if (line === '') {
                        lines.push('');
                        return lines;
                    }
                    return lines.concat(multilineTextLayout(line, process.stdout.columns - lineSpacer.length));
                }, []).map(line => lineSpacer + line),
                ''
            ];
            render(lines);
            break;
        }
    }
}
const renderState = {
    cursorY: 1
};
function clear() {
    cursorTo(1, 1);
    process.stdout.write(`\x1b[0J`);
}
function render(lines) {
    process.stdout.write(lines.join('\n') + '\n');
    renderState.cursorY = lines.length + 1;
}
function cursorTo(x, y) {
    const yDelta = renderState.cursorY - y;
    if (yDelta > 0) {
        process.stdout.write(`\x1b[${yDelta}A`);
    }
    process.stdout.write(`\x1b[${x}G`);
    renderState.cursorY = y;
}
const CTRL_C = Buffer.from('03', 'hex');
const UP = Buffer.from('1b5b41', 'hex');
const DOWN = Buffer.from('1b5b42', 'hex');
const RIGHT = Buffer.from('1b5b43', 'hex');
const LEFT = Buffer.from('1b5b44', 'hex');
const DELETE = Buffer.from('7f', 'hex');
const BACKSPACE = Buffer.from('08', 'hex');
const ENTER = Buffer.from('0d', 'hex');
function log(s) {
    fs_1.appendFileSync('./log', Buffer.from(`${JSON.stringify(s)}\n`));
}
const escapeCode = 0x1b;
const UNICODE_C0_RANGE = [0x00, 0x1f];
const UNICODE_C1_RANGE = [0x80, 0x9f];
function isEscapeCode(data) {
    return data[0] === escapeCode;
}
function isC0C1ControlCode(data) {
    if (data.length > 1) {
        return false;
    }
    const code = data[0];
    const inC0Range = code >= UNICODE_C0_RANGE[0] && code <= UNICODE_C0_RANGE[1];
    const inC1Range = code >= UNICODE_C1_RANGE[0] && code <= UNICODE_C1_RANGE[1];
    return inC0Range || inC1Range;
}
function isDeleteKey(data) {
    return data.length === 1 && data[0] === DELETE[0];
}
function isMetaPlusNumberCombination(key) {
    if (key.length === 2 && key[0] === escapeCode) {
        return key[1] >= 0x30 && key[1] <= 0x39;
    }
}
function getNumberFromMetaPlusCombination(key) {
    return key[1] & 0x0F;
}
function isSpecialKey(key) {
    return isEscapeCode(key) || isC0C1ControlCode(key) || isDeleteKey(key);
}
var ListSortCriterion;
(function (ListSortCriterion) {
    ListSortCriterion[ListSortCriterion["LastSwitch"] = 0] = "LastSwitch";
    ListSortCriterion[ListSortCriterion["SearchMatchScore"] = 1] = "SearchMatchScore";
})(ListSortCriterion || (ListSortCriterion = {}));
function sortedListLines(list, criterion) {
    if (criterion === ListSortCriterion.LastSwitch) {
        return list.slice().sort((a, b) => {
            if (b.type === ListLineType.Head) {
                return 1;
            }
            return b.content.lastSwitch - a.content.lastSwitch;
        });
    }
    return list.slice().sort((a, b) => {
        return b.searchMatchScore - a.searchMatchScore;
    });
}
function generateList(state) {
    let list = [];
    list.push({
        type: ListLineType.Head,
        content: state.currentHEAD,
        searchMatchScore: state.searchString === '' ? 1 : fuzzy_1.fuzzyMatch(state.searchString, state.currentHEAD.detached ? state.currentHEAD.sha : state.currentHEAD.branchName)
    });
    const branchLines = state.branches
        .filter(branch => {
        return (state.currentHEAD.detached
            || branch.name !== state.currentHEAD.branchName);
    })
        .map((branch) => {
        return {
            type: ListLineType.Branch,
            content: branch,
            searchMatchScore: state.searchString === '' ? 1 : fuzzy_1.fuzzyMatch(state.searchString, branch.name)
        };
    });
    list = list.concat(branchLines)
        .filter((line) => line.searchMatchScore > 0);
    const sortCriterion = state.searchString === '' ? ListSortCriterion.LastSwitch : ListSortCriterion.SearchMatchScore;
    return sortedListLines(list, sortCriterion);
}
function locateGitRepoFolder(folder) {
    const dir = fs_1.opendirSync(folder);
    let item = dir.readSync();
    let found = false;
    while (item !== null && !found) {
        found = item.isDirectory() && item.name === '.git';
        item = dir.readSync();
    }
    dir.closeSync();
    if (found) {
        return folder;
    }
    if (folder === '/') {
        throw new InputError(`You're not in Git repo.`, 'There is no Git repository in current or any parent folder.');
    }
    return locateGitRepoFolder(fsPath.resolve(folder, '..'));
}
function readVersion() {
    return JSON.parse(fs_1.readFileSync(fsPath.join(__dirname, '../package.json')).toString()).version;
}
function readRawGitBranches(gitRepoFolder) {
    function collectBranchNames(folderPath, prefix = '') {
        return fs_1.readdirSync(folderPath, { withFileTypes: true })
            .reduce((branches, item) => {
            if (item.isFile()) {
                branches.push(prefix + item.name);
            }
            if (item.isDirectory()) {
                branches = branches.concat(collectBranchNames(fsPath.join(folderPath, item.name), `${prefix}${item.name}/`));
            }
            return branches;
        }, []);
    }
    return collectBranchNames(fsPath.join(gitRepoFolder, '.git/refs/heads'));
}
const JUMP_FOLDER = '.jump';
const DATA_FILE_PATH = `${JUMP_FOLDER}/data.json`;
function readBranchesJumpData(gitRepoFolder) {
    try {
        return JSON.parse(fs_1.readFileSync(fsPath.join(gitRepoFolder, DATA_FILE_PATH)).toString());
    }
    catch (e) {
        throw new Error(`JSON in "${DATA_FILE_PATH}" is not valid, could not parse it.`);
    }
}
function saveBranchesJumpData(gitRepoFolder, jumpData) {
    try {
        fs_1.writeFileSync(fsPath.join(gitRepoFolder, DATA_FILE_PATH), JSON.stringify(jumpData, null, 2));
    }
    catch (e) {
        throw new Error(`Could not write data into "${DATA_FILE_PATH}".`);
    }
}
function cleanUpJumpData(gitRepoFolder, jumpData, rawGitBranches) {
    const cleanJumpData = Object.keys(jumpData).reduce((cleanData, jumpDataBranchName) => {
        if (rawGitBranches.includes(jumpDataBranchName)) {
            cleanData[jumpDataBranchName] = jumpData[jumpDataBranchName];
        }
        return cleanData;
    }, {});
    saveBranchesJumpData(gitRepoFolder, cleanJumpData);
}
function readBranchesData(gitRepoFolder) {
    const rawGitBranches = readRawGitBranches(gitRepoFolder);
    const branchesJumpData = readBranchesJumpData(gitRepoFolder);
    cleanUpJumpData(gitRepoFolder, branchesJumpData, rawGitBranches);
    return rawGitBranches
        .map(branch => {
        const jumpData = branchesJumpData[branch];
        return {
            name: branch,
            lastSwitch: jumpData !== undefined ? jumpData.lastSwitch : 0
        };
    });
}
function updateBranchLastSwitch(name, lastSwitch, state) {
    const jumpData = readBranchesJumpData(state.gitRepoFolder);
    jumpData[name] = { name, lastSwitch };
    saveBranchesJumpData(state.gitRepoFolder, jumpData);
}
function renameJumpDataBranch(currentName, newName, state) {
    const jumpData = readBranchesJumpData(state.gitRepoFolder);
    const currentJumpData = jumpData[currentName];
    if (currentJumpData === undefined) {
        return;
    }
    jumpData[newName] = Object.assign(Object.assign({}, currentJumpData), { name: newName });
    delete jumpData[currentName];
    saveBranchesJumpData(state.gitRepoFolder, jumpData);
}
function deleteJumpDataBranch(branchNames, state) {
    const jumpData = readBranchesJumpData(state.gitRepoFolder);
    branchNames.forEach((name) => {
        if (jumpData[name] === undefined) {
            return;
        }
        delete jumpData[name];
    });
    saveBranchesJumpData(state.gitRepoFolder, jumpData);
}
function readCurrentHEAD(gitRepoFolder) {
    const head = fs_1.readFileSync(fsPath.join(gitRepoFolder, '.git/HEAD')).toString();
    const detached = !head.startsWith('ref:');
    return {
        detached,
        sha: detached ? head.slice(0, 7).trim() : null,
        branchName: detached ? null : head.slice(16).trim()
    };
}
function getBranchNameForLine(line) {
    switch (line.type) {
        case ListLineType.Head: {
            const content = line.content;
            return content.detached ? content.sha : content.branchName;
        }
        case ListLineType.Branch: {
            return line.content.name;
        }
    }
}
function gitCommand(command, args) {
    const commandString = ['git', command, ...args].join(' ');
    const { stdout, stderr, error, status } = child_process_1.spawnSync('git', [command, ...args]);
    if (error) {
        throw new Error(`Could not run ${bold(commandString)}.`);
    }
    const cleanLines = (stream) => stream.toString().trim().split('\n').filter(line => line !== '');
    const statusIndicatorColor = status > 0 ? red : green;
    const message = [
        statusIndicatorColor('‣ ') + dim(commandString),
        ...cleanLines(stdout),
        ...cleanLines(stderr)
    ];
    log(message);
    return { status, message };
}
function switchBranch(branchName, state) {
    const switchResult = gitCommand('switch', [branchName]);
    if (switchResult.status === 0) {
        updateBranchLastSwitch(branchName, Date.now(), state);
    }
    return switchResult;
}
function selectListLine(line) {
    const branchName = getBranchNameForLine(line);
    if (line.type === ListLineType.Head) {
        state.scene = Scene.Message;
        state.message = [`Staying on ${bold(branchName)}`];
        view(state);
        process.exit(0);
    }
    const { status, message } = switchBranch(branchName, state);
    state.scene = Scene.Message;
    state.message = message;
    view(state);
    process.exit(status);
}
function handleSpecialKey(key) {
    if (key.equals(CTRL_C)) {
        clear();
        process.exit();
    }
    if (key.equals(ENTER)) {
        selectListLine(state.list[state.highlightedLineIndex]);
        return;
    }
    if (key.equals(UP)) {
        state.highlightedLineIndex = Math.max(0, state.highlightedLineIndex - 1);
        view(state);
        return;
    }
    if (key.equals(RIGHT)) {
        if (state.searchStringCursorPosition === state.searchString.length) {
            return;
        }
        state.searchStringCursorPosition += 1;
        view(state);
        return;
    }
    if (key.equals(LEFT)) {
        if (state.searchStringCursorPosition === 0) {
            return;
        }
        state.searchStringCursorPosition -= 1;
        view(state);
        return;
    }
    if (key.equals(DOWN)) {
        state.highlightedLineIndex = Math.min(state.list.length - 1, state.highlightedLineIndex + 1);
        view(state);
        return;
    }
    if (key.equals(DELETE) || key.equals(BACKSPACE)) {
        if (state.searchStringCursorPosition === 0) {
            return;
        }
        state.searchString = state.searchString.substring(0, state.searchStringCursorPosition - 1) + state.searchString.substring(state.searchStringCursorPosition, state.searchString.length);
        state.searchStringCursorPosition -= 1;
        state.list = generateList(state);
        state.highlightedLineIndex = 0;
        view(state);
        return;
    }
    if (isMetaPlusNumberCombination(key)) {
        const quickSelectIndex = getNumberFromMetaPlusCombination(key);
        const quickSelectLines = getQuickSelectLines(state.list);
        if (quickSelectIndex < quickSelectLines.length) {
            selectListLine(quickSelectLines[quickSelectIndex]);
        }
        return;
    }
}
function handleStringKey(key) {
    const inputString = key.toString();
    state.searchString = state.searchString.substring(0, state.searchStringCursorPosition) + inputString + state.searchString.substring(state.searchStringCursorPosition, state.searchString.length);
    state.searchStringCursorPosition += inputString.length;
    state.list = generateList(state);
    state.highlightedLineIndex = 0;
    view(state);
}
function bare() {
    view(state);
    if (!state.isInteractive) {
        process.exit(0);
    }
    process.stdin.setRawMode(true);
    process.stdin.on('data', (data) => {
        parseKeys_1.parseKeys(data).forEach((key) => {
            if (isSpecialKey(key)) {
                handleSpecialKey(key);
                return;
            }
            handleStringKey(key);
        });
    });
}
function jumpTo(args) {
    const switchResult = gitCommand('switch', args);
    if (switchResult.status === 0) {
        state.scene = Scene.Message;
        state.message = switchResult.message;
        view(state);
        process.exit(0);
    }
    state.searchString = args[0];
    state.list = generateList(state);
    if (state.list.length === 0) {
        state.scene = Scene.Message;
        state.message = [`${bold(yellow(state.searchString))} does not mach any branch`];
        view(state);
        process.exit(1);
    }
    selectListLine(state.list[0]);
}
function multilineTextLayout(text, columns) {
    if (text.length === 0) {
        return [];
    }
    const words = text.split(' ');
    const escapeCodePattern = /\x1b.+?m/gi;
    return words.slice(1).reduce((lines, word) => {
        const currentLine = lines[lines.length - 1];
        const sanitizedCurrentLine = currentLine.replace(escapeCodePattern, '');
        const sanitizedWord = word.replace(escapeCodePattern, '');
        if (sanitizedCurrentLine.length + sanitizedWord.length + 1 <= columns) {
            lines[lines.length - 1] = currentLine + ' ' + word;
        }
        else {
            lines.push(word);
        }
        return lines;
    }, [words[0]]);
}
function checkUpdates() {
    const VERSION_PATTERN = /^\d+\.\d+\.\d+$/;
    child_process_1.exec('npm info git-jump dist-tags.latest', (error, stdout) => {
        if (error) {
            return;
        }
        const output = stdout.trim();
        if (!VERSION_PATTERN.test(output)) {
            return;
        }
        state.latestPackageVersion = output;
    });
}
function compareSemver(a, b) {
    return a.localeCompare(b, undefined, { numeric: true });
}
function handleError(error) {
    if (error instanceof InputError) {
        state.message = [`${yellow(error.title)} ${error.message}`];
    }
    else {
        state.message = [
            `${red('Error:')} ${error.message}`,
            '',
            `${bold('What to do?')}`,
            'Help improve git-jump, create GitHub issue with this error and steps to reproduce it. Thank you!',
            '',
            `GitHub Issues: https://github.com/mykolaharmash/git-jump/issues`
        ];
    }
    state.scene = Scene.Message;
    view(state);
    process.exit(1);
}
function handleExit() {
    if (state.latestPackageVersion === null) {
        return;
    }
    const currentVersion = readVersion();
    if (compareSemver(currentVersion, state.latestPackageVersion) === -1) {
        const sourcePackageManager = fs_1.existsSync(fsPath.join(__dirname, '../homebrew')) ? 'homebrew' : 'npm';
        const updateCommand = sourcePackageManager === 'npm' ? 'npm install -g git-jump' : 'brew upgrade git-jump';
        state.scene = Scene.Message;
        state.message = state.message.concat([
            '',
            `New version of git-jump is available: ${yellow(currentVersion)} → ${green(state.latestPackageVersion)}.`,
            `Changelog: https://github.com/mykolaharmash/git-jump/releases/tag/v${state.latestPackageVersion}`,
            '',
            `${bold(updateCommand)} to update.`
        ]);
        view(state);
    }
}
function initialize() {
    state.isInteractive = process.stdout.isTTY === true;
    state.gitRepoFolder = locateGitRepoFolder(process.cwd());
    const jumpFolderPath = fsPath.join(state.gitRepoFolder, JUMP_FOLDER);
    const dataFileFullPath = fsPath.join(state.gitRepoFolder, DATA_FILE_PATH);
    if (!fs_1.existsSync(jumpFolderPath)) {
        fs_1.mkdirSync(jumpFolderPath);
        fs_1.appendFileSync(fsPath.join(state.gitRepoFolder, '.git', 'info', 'exclude'), `\n${JUMP_FOLDER}`);
    }
    if (!fs_1.existsSync(dataFileFullPath)) {
        fs_1.writeFileSync(dataFileFullPath, '{}', { flag: 'a' });
    }
    state.currentHEAD = readCurrentHEAD(state.gitRepoFolder);
    state.branches = readBranchesData(state.gitRepoFolder);
    state.list = generateList(state);
    state.highlightedLineIndex = 0;
}
function main(args) {
    process.on('uncaughtException', handleError);
    process.on('exit', handleExit);
    initialize();
    if (args.length === 0) {
        checkUpdates();
        bare();
        return;
    }
    if (isSubCommand(args)) {
        executeSubCommand(args[0], args.slice(1));
        return;
    }
    jumpTo(args);
}
main(process.argv.slice(2));
//# sourceMappingURL=index.js.map